-- bucher.lua
-- Created on:  2020-10-16
-- Author:      Rétro
--
-- API d'abattage d'arbres pour tortures RSI

---
-- \class Bucheron RSI
local RSI_Bucheron = {}
RSI_Bucheron.__index = RSI_Bucheron

rsiT = require("rsiTurtle")
require("macros")

---
-- \brief Constructeur de RSI_Bucheron
function RSI_Bucheron:init()
    local bucheron = {}
    setmetatable(bucheron,RSI_Bucheron)

    ---
    -- \brief Objet parent
    bucheron.base_ = rsiT:init()

    return bucheron
end

---
-- \brief Affiche la position et l'orientation du bucheron
function RSI_Bucheron:print()
    print("Type bucheron.")
    self.base_:print()
end


---
-- \brief Abattre un arbre
-- \return bool Succès
function RSI_Bucheron:chopTree()
    local success
    local countMovedUp = 0

    success = self.base_:dig(ID_LOG)
    if (not success)
    then
        return false
    end

    success = self.base_:forward()

    print("Chopping down")
    while ( self.base_:dig(ID_LOG, "down") ) do self.base_:down() end
    print("Moving up")

    -- Remonter pour couper les bûches vers le haut. Essaie de trouver des
    -- bûches sur quelques blocs de haut avant d'abandonner et redescendre.
    while (countMovedUp < 3 and self.base_:up())
    do
        countMovedUp = countMovedUp + 1
        printf("Moved up %d\n", countMovedUp)
    end

    print("Chopping up")
    while ( self.base_:dig(ID_LOG, "up") ) do self.base_:up() end
    self.base_:ground()

    return success
end


---
-- \brief Couper les arbres tout en avançant le long d'une ligne
-- \param int   distance  Nombre de blocs à parcourir
-- \return bool Succès
function RSI_Bucheron:maintainFuelLevels()
    if (turtle.getFuelLevel() < FUEL_LOW)
    then
        return self.base_:refuelUntil(FUEL_FULL, SLOT_ENDERCHEST_FUEL)
    end
    return true
end

---
-- \brief Si l'inventaire est plein, le vider dans le enderchest prévu pour
-- \return bool Succès
function RSI_Bucheron:manageInventory()
    if ( self.base_:inventoryFull(SLOT_STORAGE_START, SLOT_STORAGE_END) )
    then
        return self.base_:dumpInventory(SLOT_STORAGE_START, SLOT_STORAGE_END,
                                        SLOT_ENDERCHEST_DUMP)
    end
end


---
-- \brief Couper les arbres dans la zone
-- \param int lines Nombre de lignes
-- \param int cols  Nombre de colonnes
-- \return bool Succès
function RSI_Bucheron:chopArea(lines, cols)
    --TODO: Implementer
    local success = true

    assert(lines >= 1)
    assert(cols >= 1)

    action = function ()
        self:maintainFuelLevels()
        self:manageInventory()
        turtle.select(SLOT_STORAGE_START)

        self:chopTree()
    end
    return self.base_:goThroughAreaGrounded(lines, cols, action)

end


return RSI_Bucheron