-- excaver.lua
-- Created on:  2020-10-17
-- Author:      Rétro
--
-- API d'excavation de terrain

---
-- \class Bucheron RSI
local RSI_Excavatrice = {}
RSI_Excavatrice.__index = RSI_Excavatrice

rsiT = require("rsiTurtle")

---
-- \brief Constructeur de RSI_Excavatrice
function RSI_Excavatrice:init()
    local excavatrice = {}
    setmetatable(excavatrice,RSI_Excavatrice)

    ---
    -- \brief Objet parent
    excavatrice.base_ = rsiT:init()

    return excavatrice
end


---
-- \brief Couper les arbres tout en avançant le long d'une ligne
-- \param int   distance  Nombre de blocs à parcourir
-- \return bool Succès
function RSI_Excavatrice:maintainFuelLevels()
    if (turtle.getFuelLevel() < FUEL_LOW)
    then
        return self.base_:refuelUntil(FUEL_FULL, SLOT_ENDERCHEST_FUEL)
    end
    return true
end

---
-- \brief Si l'inventaire est plein, le vider dans le enderchest prévu pour
-- \return bool Succès
function RSI_Excavatrice:manageInventory()
    if ( self.base_:inventoryFull(SLOT_STORAGE_START, SLOT_STORAGE_END) )
    then
        return self.base_:dumpInventory(SLOT_STORAGE_START, SLOT_STORAGE_END,
                                        SLOT_ENDERCHEST_DUMP)
    end
end


---
-- \brief Quadriller une zone carrée. La tortue traverse chaque case, ligne par
--          ligne. N'affecte pas l'altitude.
-- \param int               lines       Nombre de lignes    (direction: avant)
-- \param int               cols        Nombre de colonnes  (direction: droite)
-- \return bool Succès
function RSI_Excavatrice:excavateArea(lines, cols)
    action = function ()
        self:maintainFuelLevels()
        self:manageInventory()
        turtle.select(SLOT_STORAGE_START)
        self.base_:dig(nil, "down")
    end
    return self.base_:goThroughArea(lines, cols, action)
end


---
-- \brief Affiche la position et l'orientation de l'excavatrice
function RSI_Excavatrice:print()
    print("Type excavatrice.")
    self.base_:print()
end


---
-- \brief Dégager la zone vers le bas
-- \param int lines     Nombre de lignes
-- \param int cols      Nombre de colonnes
-- \param int depth     Profondeur z (vers le bas) sur laquelle excaver
-- \return bool Succès
function RSI_Excavatrice:excavateVolumeDown(lines, cols, depth)
    action = function ()
        self:maintainFuelLevels()
        self:manageInventory()
        turtle.select(SLOT_STORAGE_START)
        self.base_:dig(nil, "down")
    end
    return self.base_:goThroughVolume(lines, cols, depth, action)
end

return RSI_Excavatrice
